(* ------------------------------------------------------------------ *)
(* Variable sized lists.                                              *)
(* Version: 1.00                                                      *)
(* Author: Peter Kofler, www.code-cop.org                             *)
(* System: Turbo Pascal 6.01                                          *)
(* ------------------------------------------------------------------ *)
UNIT Lists;

INTERFACE

{ max number of Longints in 64kb array }
CONST MaxLongintArray = 16382;

{ List backed by array of Longints, similar Java's ArrayList<Integer> }
TYPE ListElement = Longint;
     ListElementArray = ARRAY[0..MaxLongintArray] OF ListElement;
     ListElementPtr = ^ListElementArray;
     ArrayList = OBJECT
        _size:Word;
        _capacity:Word;
        _elements:ListElementPtr;
        CONSTRUCTOR Init;
        PROCEDURE Put(index:Word; element:ListElement);
        PROCEDURE Add(element:ListElement);
        FUNCTION Get(index:Word):ListElement;
        FUNCTION IsEmpty:Boolean;
        FUNCTION Size:Word;
        DESTRUCTOR Destroy;
     END;
     ArrayListPtr = ^ArrayList;

IMPLEMENTATION

CONST step = sizeof(ListElement);

CONSTRUCTOR ArrayList.Init;
BEGIN
   _size := 0;
   _capacity := 10;
   GetMem(_elements, _capacity * step);
   FillChar(_elements^, _capacity * step, 0);
END;

PROCEDURE ArrayList.Put(index:Word; element:ListElement);
VAR oldElements:ListElementPtr;
    oldCapacity:Word;
BEGIN
   IF index >= _capacity THEN
   BEGIN
      { ensure capacity }
      oldCapacity := _capacity;
      oldElements := _elements;
      WHILE _capacity <= index DO
      BEGIN
         _capacity := _capacity * 2;
      END;
      GetMem(_elements, _capacity * step);
      FillChar(_elements^, _capacity * step, 0);
      Move(oldElements^, _elements^, oldCapacity * step);
      FreeMem(oldElements, oldCapacity * step);
   END;
   _elements^[index] := element;
   IF index >= _size THEN
   BEGIN
      _size := index + 1;
   END;
END;

PROCEDURE ArrayList.Add(element:ListElement);
BEGIN
   Put(_size, element);
END;

FUNCTION ArrayList.Get(index:Word):ListElement;
BEGIN
   IF index > _size THEN
   BEGIN
      RunError;
   END;
   get := _elements^[index];
END;

FUNCTION ArrayList.IsEmpty:Boolean;
BEGIN
   isEmpty := (size = 0);
END;

FUNCTION ArrayList.Size:Word;
BEGIN
   size := _size;
END;

DESTRUCTOR ArrayList.Destroy;
BEGIN
   FreeMem(_elements, _capacity * step);
END;

END.
